/* TiniSyslogD - Tiny Syslog Server for TINI
 * Copyright(C) 2002  Yuichi Kobayashi <kobayasy@kobayasy.com>
 *
 * This program comes with NO WARRANTY, to the extent permitted by law.
 * You may redistribute it under the terms of the GNU General Public License;
 * see the file named LICENSE.txt for details.
 */

package com.kobayasy.tini;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 * Tiny Syslog Server for TINI
 *
 * @version 0.7 (20021101)
 * @author  Yuichi Kobayashi &lt;kobayasy@kobayasy.com&gt;
 */
public class TiniSyslogD extends Thread {
  public static final String CLASSNAME = "TiniSyslogD";
  public static final String VERSION = "0.7";
  public static final String CLASSINFO = CLASSNAME + " version " + VERSION +
    " - Tiny Syslog Server for TINI\r\n" +
    "Copyright(C) 2002  Yuichi Kobayashi <kobayasy@kobayasy.com>\r\n" +
    "This program comes with NO WARRANTY, to the extent permitted by law.\r\n" +
    "You may redistribute it under the terms of the GNU General Public License;\r\n";
  private TiniSyslogDConfig config;
  private TiniSyslog syslog;
  public TiniSyslogD(TiniSyslogDConfig config, TiniSyslog syslog) {
    this.config = config;
    this.syslog = syslog;
  }
  public void run() {
    try {
      DatagramSocket socket = null;
      try {
        socket = new DatagramSocket(config.accept_port);
        byte[] buffer = new byte[config.buffer_size];
        while(true) {
          DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
          socket.receive(packet);
          System.out.println(getDate() + " " + packet.getAddress().getHostName() + " " + new String(packet.getData(), 0, packet.getLength()));
        }
      } finally {
        if(socket != null)
          socket.close();
      }
    } catch(IOException e) {
      if(syslog != null)
        syslog.sendMessage("error: " + e);
    }
  }
  private String getDate() {
    Date date = new Date();
    StringBuffer buffer = new StringBuffer();
    StringTokenizer tokens = new StringTokenizer(date.toString(), " ");
    for(int n = 0; tokens.hasMoreTokens(); ++n) {
      String token = tokens.nextToken();
      switch(n) {
       case 2:
       case 3:
        buffer.append(' ');
       case 1:
        buffer.append(token);
        break;
      }
    }
    return buffer.toString();
  }
//
  public static void main(String[] args) {
    TiniSyslogDConfig config = new TiniSyslogDConfig();
    boolean status = config.setParameter(TiniSyslogDConfig.DEFAULT);
    if(status)
      for(int n = 0; n < args.length; ++n)
        if(!config.setParameter(args[n]))
          status = false;
    if(status) {
      System.out.println("Starting " + CLASSNAME + " ...");
      TiniSyslog syslog = null;
      try {
        if(config.syslog_host != null)
          try {
            syslog = new TiniSyslog(config.syslog_host, config.syslog_port);
            syslog.setPrefix(CLASSNAME);
          } catch(UnknownHostException e) {
            System.err.println(e);
          } catch(SocketException e) {
            System.err.println(e);
          }
        try {
          if(syslog != null)
            syslog.sendMessage("start: " + config.toString().replace('\n', ' ').trim());
          TiniSyslogD httpd = new TiniSyslogD(config, syslog);
          httpd.setDaemon(true);
          httpd.start();
          httpd.join();
        } catch(InterruptedException e) {
          if(syslog != null)
            syslog.sendMessage("interrupt: " + e);
        }
      } catch(Throwable e) {
        if(syslog != null)
          syslog.sendMessage("error: " + e);
      } finally {
        if(syslog != null)
          syslog.close();
      }
    }
  }
}
