/* TiniSyslogDConfig - Configuration of TiniSyslogD
 * Copyright(C) 2002  Yuichi Kobayashi <kobayasy@kobayasy.com>
 *
 * This program comes with NO WARRANTY, to the extent permitted by law.
 * You may redistribute it under the terms of the GNU General Public License;
 * see the file named LICENSE.txt for details.
 */

package com.kobayasy.tini;
import java.io.*;

/**
 * Configuration of TiniSyslogD
 *
 * @version 20021022
 * @author  Yuichi Kobayashi &lt;kobayasy@kobayasy.com&gt;
 */
public class TiniSyslogDConfig {
  public int accept_port = 512;
  public int buffer_size = 1024;
  public String syslog_host = null;
  public int syslog_port = 512;
  public static final String INFO     = "info";
  public static final String HELP     = "help";
  public static final String HELP1    = "-help";
  public static final String HELP2    = "--help";
  public static final String HELP3    = "-h";
  public static final String DEFAULT  = "default";
  public static final String PORT     = "port=";
  public static final String BUFFER   = "buffer=";
  public static final String SYSLOG   = "syslog=";
  public boolean setParameter(String arg) {
    boolean status;
    if(arg.equals(INFO)) {
      System.out.print(TiniSyslogD.CLASSINFO);
      status = false;
    }
    else if(arg.equals(HELP) ||
            arg.equals(HELP1) ||
            arg.equals(HELP2) ||
            arg.equals(HELP3) ) {
      System.out.print(getUsage());
      status = false;
    }
    else if(arg.equals(DEFAULT))
      status = setDefault();
    else if(arg.startsWith(PORT))
      try {
        accept_port = Integer.parseInt(arg.substring(PORT.length()));
        status = true;
      } catch(NumberFormatException e) {
        System.err.println("Invalid value: " + arg);
        status = false;
      }
    else if(arg.startsWith(BUFFER))
      try {
        buffer_size = Integer.parseInt(arg.substring(BUFFER.length()));
        status = true;
      } catch(NumberFormatException e) {
        System.err.println("Invalid value: " + arg);
        status = false;
      }
    else if(arg.equals(SYSLOG + File.pathSeparatorChar)) {
      syslog_host = null;
      status = true;
    }
    else if(arg.startsWith(SYSLOG)) {
      int n1 = SYSLOG.length();
      int n2 = arg.indexOf(File.pathSeparatorChar, n1);
      if(n2 != -1) {
        syslog_host = arg.substring(n1, n2);
        try {
          syslog_port = Integer.parseInt(arg.substring(n2 + 1));
          status = true;
        } catch(NumberFormatException e) {
          System.err.println("Invalid value: " + arg);
          status = false;
        }
      }
      else {
        syslog_host = arg.substring(n1);
        syslog_port = 512;
        status = true;
      }
    }
    else {
      System.err.println("Invalid parameter: " + arg);
      status = false;
    }
    return status;
  }
  public String toString() {
    StringBuffer s = new StringBuffer();
    s.append(PORT + accept_port + '\n');
    s.append(BUFFER + buffer_size + '\n');
    if(syslog_host != null)
      s.append(SYSLOG + syslog_host + File.pathSeparatorChar + syslog_port + '\n');
    return s.toString();
  }
  private String getUsage() {
    StringBuffer usage = new StringBuffer();
    usage.append("Usage: " + TiniSyslogD.CLASSNAME);
    usage.append(" [" + INFO + "]");
    usage.append(" [" + HELP + "]");
    usage.append(" [" + PORT + "accept-port]");
    usage.append(" [" + BUFFER + "buffer-size]");
    usage.append(" [" + SYSLOG + "syslog-host[" + File.pathSeparatorChar + "syslog-port]]");
    usage.append("\r\n");
    usage.append("Current configuration: " + toString().replace('\n', ' ').trim() + "\r\n");
    return usage.toString();
  }
  private boolean setDefault() {
    return setParameter(PORT     + "512") &&
           setParameter(BUFFER   + "1024") &&
           setParameter(SYSLOG   + File.pathSeparatorChar);
  }
}
